/*
Copyright © 2023 Philipp Wolfer <phw@uploadedlobster.com>

Permission is hereby granted, free of charge, to any person obtaining a copy
of this software and associated documentation files (the "Software"), to deal
in the Software without restriction, including without limitation the rights
to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
copies of the Software, and to permit persons to whom the Software is
furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in
all copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
THE SOFTWARE.
*/
package listenbrainz

import (
	"time"
)

type GetListensResult struct {
	Payload GetListenPayload `json:"payload"`
}

type GetListenPayload struct {
	Count                 int      `json:"count"`
	UserName              string   `json:"user_id"`
	LatestListenTimestamp int64    `json:"latest_listen_ts"`
	Listens               []Listen `json:"listens"`
}

type Listen struct {
	InsertedAt    int64  `json:"inserted_at"`
	ListenedAt    int64  `json:"listened_at"`
	RecordingMsid string `json:"recording_msid"`
	UserName      string `json:"user_name"`
	TrackMetadata Track  `json:"track_metadata"`
}

type Track struct {
	TrackName      string         `json:"track_name"`
	ArtistName     string         `json:"artist_name"`
	ReleaseName    string         `json:"release_name"`
	AdditionalInfo map[string]any `json:"additional_info"`
}

func (t Track) Duration() time.Duration {
	var duration time.Duration
	milliseconds, ok := t.AdditionalInfo["duration_ms"].(float64)
	if ok {
		duration = time.Duration(int64(milliseconds) * int64(time.Millisecond))
	} else {
		seconds, ok := t.AdditionalInfo["duration_ms"].(float64)
		if ok {
			duration = time.Duration(int64(seconds) * int64(time.Second))
		}
	}
	return duration
}

func (t Track) TrackNumber() int {
	return int(tryGetValue[float64](t, "tracknumber"))
}

func (t Track) Isrc() string {
	return tryGetValue[string](t, "isrc")
}

func (t Track) RecordingMbid() string {
	return tryGetValue[string](t, "recording_mbid")
}

func (t Track) ReleaseMbid() string {
	return tryGetValue[string](t, "release_mbid")
}

func (t Track) ReleaseGroupMbid() string {
	return tryGetValue[string](t, "release_group_mbid")
}

func tryGetValue[T any](track Track, key string) T {
	var result T
	value, ok := track.AdditionalInfo[key].(T)
	if ok {
		result = value
	}
	return result
}
